// headers.h
//
// header set

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "headers.h"
#include "utility.h"

Headers::Headers()
{
	mParms = NULL;
}

Headers::~Headers()
{
	Header*	hp = mParms;

	while (hp)
	{
		Header*	next = hp->mNext;
		delete[] hp->mName;
		delete hp;
		hp = next;
	}
}

void Headers::AddHeader(const char* name, const char* data)
{
	Header*	hp = new Header;

	hp->mName = new char[strlen(name) + 1];
	strcpy(hp->mName, name);

	hp->mData = new char[strlen(data) + 1];
	strcpy(hp->mData, data);

	hp->mNext = mParms;
	mParms = hp;
}

void Headers::AddFromHTTP(const char* httprq)
{
	if (!httprq)	return;

	char*	hdr = StringDup(httprq);

	for (;;)
	{
		char*	key;

		// split at <LF>
		hdr = StringSplit(hdr, 0x0A, key);

		// no more keys?
		if (!key || !key[0])	break;

		// scan for colon
		char* hptr = key;
		while (*hptr && (*hptr != ':'))	hptr++;

		// add header
		if (*hptr)
		{
			*hptr++ = '\0';
			while (*hptr == ' ')	hptr++;
			AddHeader(key, hptr);
		}
		else
		{
			AddHeader(key, "");
		}

		delete[] key;
	}

	delete[] hdr;
}

void Headers::AddFromCGI(const char* cgistr)
{
	if (!cgistr)	return;

	char*	data = StringDup(cgistr);

	while (data)
	{
		char*	line;

		// split at &, else take remainder
		data = StringSplit(data, '&', line);
		if (!line)
		{
			line = data;
			data = NULL;
		}

		// split at =
		char*	key;

		line = StringSplit(line, '=', key);
		if (key)
		{
			// parse first
			URLDecode(line);
			AddHeader(key, line);
		}
		else if (line[0])
		{
			AddHeader(line, "");
		}

		delete[] line;
		delete[] key;
	}
}

const Header* Headers::Find(const char* name) const
{
	for (Header* hp = mParms; hp; hp = hp->mNext)
	{
		if (!stricmp(hp->mName, name))	return hp;
	}

	return NULL;
}

void Headers::Write(FILE* fd) const
{
	for (Header* hp = mParms; hp; hp = hp->mNext)
	{
		fprintf(fd, "%s: %s\n", hp->mName, hp->mData);
	}
}

int Headers::Count() const
{
	int	count = 0;

	for (Header* hp = mParms; hp; hp = hp->mNext)	count++;

	return count;
}
