// httprq.cpp
//
// HTTP_RQ object

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "httprq.h"
#include "utility.h"

HTTP_RQ::HTTP_RQ()
{
	Clear();
}

HTTP_RQ::~HTTP_RQ()
{
	Destroy();
}

// Clear
//
// clear an HTTP_RQ

void HTTP_RQ::Clear()
{
	memset(this, 0, sizeof(*this));
}

// Destroy
//
// destroy an HTTP_RQ

void HTTP_RQ::Destroy()
{
	delete[] _req;
	delete[] _uri;
	delete[] _httpv;

	delete _hdrs;
	delete _uridata;
	delete _postdata;

	Clear();
}

// ValidateCommand
//
// validate a command

bool HTTP_RQ::ValidateCommand()
{
	static char* cmds[] =
	{
		"get","head",
		"post",
		"checkin","checkout",
		"showmethod",
		"put",
		"delete",
		"link","unlink",
		"textsearch","search",
		"spacejump",
		NULL
	};
	char**	cmdptr = cmds;

	while (*cmdptr)
	{
		if (!stricmp(*cmdptr, _req))	return true;
		cmdptr++;
	}

	return false;
}

// AttachHeader
//
// create full request from headers

void HTTP_RQ::AttachHeaders(char* hdr)
{
	// destroy old data
	Destroy();

	// get request line
	char*	rq;
	hdr = StringSplit(hdr, 0x0A, rq);

	if (!rq)	return;

	// get request code
	rq = StringSplit(rq, ' ', _req);
	if (_req)
	{
		rq = StringSplit(rq, ' ', _uri);
		if (_uri)
		{
			_httpv = rq;
		}
		else
		{
			_uri = rq;
		}
	}
	else
	{
		_req = rq;
	}

	if (!_uri)		_uri = StringDup("/");
	if (!_httpv)	_httpv = StringDup("HTTP/1.0");

	// check command is valid
	if (!ValidateCommand())
	{
		Destroy();
		return;
	}

	// look for URI data
	_uridata = new Headers;
	char*	uptr = _uri;
	while (*uptr && (*uptr != '?'))	uptr++;
	if (*uptr)
	{
		*uptr++ = 0;	// terminate URI
		_uridata->AddFromCGI(uptr);
	}

	// get extra headers
	_hdrs = new Headers;
	_hdrs->AddFromHTTP(hdr);

	return;
}

// AttachData
//
// attach a data buffer

void HTTP_RQ::AttachData(char* data, int datalen)
{
	char*	dnt = new char[datalen + 1];
	memcpy(dnt, data, datalen);
	delete[] data;

	if ((datalen > 1) && (dnt[datalen - 1] == 0x0A))	datalen--;
	if ((datalen > 1) && (dnt[datalen - 1] == 0x0D))	datalen--;
	dnt[datalen] = 0;

	delete _postdata;
	_postdata = new Headers;
	_postdata->AddFromCGI(dnt);
}

// GetHeader
//
// get value by key

const char* HTTP_RQ::GetHeader(const char* key) const
{
	if (!_hdrs)	return NULL;

	const Header*	hdr = _hdrs->Find(key);

	if (!hdr)	return NULL;

	return hdr->mData;
}

// ConvertGetToPost
//
// replace Post data with the URI data, if possible

bool HTTP_RQ::ConvertGetToPost()
{
	if (_postdata->Count())	return false;

	delete _postdata;
	_postdata = _uridata;
	_uridata = new Headers;

	return true;
}

// StatusLine
//
// convert number to status line

static struct
{
	int		code;
	char*	desc;
}
code_table[] =
{
	200, "HTTP/1.0 200 OK",
	201, "HTTP/1.0 201 Created",
	202, "HTTP/1.0 202 Accepted",
	203, "HTTP/1.0 203 Partial information",
	204, "HTTP/1.0 204 No response",

	301, "HTTP/1.0 301 Moved",
	302, "HTTP/1.0 302 Found",
	303, "HTTP/1.0 303 Method",
	304, "HTTP/1.0 304 Not modified",

	400, "HTTP/1.0 400 Bad request",
	401, "HTTP/1.0 401 Unauthorized",
	402, "HTTP/1.0 402 Payment required",
	403, "HTTP/1.0 403 Forbidden",
	404, "HTTP/1.0 404 Not found",
	
	500, "HTTP/1.0 500 Internal error",
	501, "HTTP/1.0 501 Not implemented",
	502, "HTTP/1.0 502 Server temporarily overloaded",
	503, "HTTP/1.0 503 Gateway timeout",

	0, NULL
};

const char* StatusLine(int status)
{
	for (int ii = 0; code_table[ii].code; ii++)
	{
		if (code_table[ii].code == status)	return code_table[ii].desc;
	}

	return "HTTP/1.0 500 Internal error";
}
