// main.cpp
//
// main function for Wiki server

#include <stdio.h>
#include <stdlib.h>

#include "server.h"

#include "wiki\page.h"
#include "wiki\wikiserver.h"

static int		port = -1;			// server port
static bool		clearlog = false;	// clear log
static char*	logfile = NULL;		// log filename

static void parse_cmdline(int argc, char** argv);

int main(int argc, char** argv)
{
	// disable retry dialog
	SetProcessShutdownParameters(0x100, SHUTDOWN_NORETRY);

	// get command-line parameters
	parse_cmdline(argc, argv);

	// handle defaults
	if (port == -1)	port = 80;

	FILE*	logfd;

	if (logfile)
	{
		logfd = fopen(logfile, clearlog ? "w" : "a");
		if (!logfd)
		{
			fprintf(stderr, "Can't open logfile %s\n", logfile);
			exit(1);
		}
		FreeConsole();
	}
	else
	{
		logfd = stdout;
	}

	//Z+
	char* mPath = new char[strlen(argv[0])+1];
	strcpy(mPath, argv[0]);
	strcpy(strrchr(mPath, '\\') + 1, "Pages");

	// create server object
	Server*	server = new WikiServer(port, logfd, mPath);
	delete[] mPath;

	// run server
	server->Run();

	// it's an error for this to return
	return 1;
}

static void parse_cmdline(int argc, char** argv)
{
	// go through arguments
	argc--;
	argv++;

	while (argc)
	{
		char*	arg = argv[0];

		if (arg[0] == '-')
		{
			if (arg[1] == 'p')
			{
				// -p<n> = set port
				if (port != -1)
				{
					fprintf(stderr, "Port already set to %d\n", port);
					exit(1);
				}
				port = atoi(arg + 2);
				if ((port < 0) || (port > 65535))
				{
					fprintf(stderr, "Port %d is not valid\n", port);
					exit(1);
				}
			}
			else if (arg[1] == 'l')
			{
				// -l<name> = set log filename
				if (logfile != NULL)
				{
					fprintf(stderr, "Log file already set to %s\n", logfile);
					exit(1);
				}
				logfile = arg + 2;
			}
			else if (arg[1] == 'c')
			{
				// -c = clear log file
				clearlog = true;
			}
			else break;
		}
		else
		{
			break;
		}

		argc--;
		argv++;
	}

	if (argc)
	{
		fprintf(stderr, "Syntax:\n\nWikiServer [-p<port>] [-l<logfile> [-c]]\n\n");
		fprintf(stderr, "<port> = port number (0 - 65535)\n<logfile> = log filename (defaults to stdout)\n");
		fprintf(stderr, "-c = clear logfile first\n");
		exit(1);
	}
}
