// utility.cpp
//
// utilities

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <comdef.h>

//K+
inline BYTE toHex(const BYTE &x)
{
	return x > 9 ? x + 55: x + 48;
}

char* StringSplit(char* text, char at, char*& part)
{
	char*	tptr = text;
	while (*tptr && (*tptr != at))	tptr++;
	if (!*tptr)
	{
		part = NULL;
		return text;
	}

	// check for [<0D>]<0A>
	if ((at == 0x0A) && (tptr > text) && (tptr[-1] == 0x0D))
	{
		part = new char[tptr - text];
		memcpy(part, text, tptr - text - 1);
		part[tptr - text - 1] = '\0';
	}
	else
	{
		part = new char[tptr - text + 1];
		memcpy(part, text, tptr - text);
		part[tptr - text] = '\0';
	}

	tptr++;

	char*	ntext = new char[strlen(tptr) + 1];
	strcpy(ntext, tptr);
	delete[] text;

	return ntext;
}

char* StringDup(const char* str)
{
	char*	nstr = new char[strlen(str) + 1];
	strcpy(nstr, str);
	return nstr;
}

char* StringDup(const char* start, const char* end)
{
	char*	text = new char[end - start + 1];
	memcpy(text, start, end - start);
	text[end - start] = 0;
	return text;
}

static int hex(char c)
{
	if ((c >= 'A') && (c <= 'Z'))	return c - 'A' + 0x0A;
	if ((c >= 'a') && (c <= 'z'))	return c - 'a' + 0x0a;
	if ((c >= '0') && (c <= '9'))	return c - '0' + 0x00;

	return 0;
}

void URLDecode(char*& data)
{
	char*	obuf = new char[strlen(data) + 1];
	char*	optr = obuf;

	char*	dptr = data;

	while (*dptr)
	{
		if (*dptr == '+')
		{
			*optr++ = ' ';
			dptr++;
		}
		else if (*dptr == '%')
		{
			if (!dptr[1] || !dptr[2])	break;
			*optr++ = hex(dptr[1]) * 0x10 + hex(dptr[2]);
			dptr += 3;
		}
		else
		{
			*optr++ = *dptr++;
		}
	}

	*optr++ = 0;
	delete[] data;
	data = obuf;
}

void URLEncode(char*& data)
{
	static char* hdig = "0123456789abcdef";
	char*	obuf = new char[strlen(data) * 3 + 1];
	char*	optr = obuf;
	char*	dptr = data;
/*
	// do encoding
	while (*dptr)
	{
	    if(isalnum(*dptr))
	        *optr++ = *dptr;
	    else
	        if(isspace(*dptr))
		    *optr++ = '+';
		else
		{
		    *optr++ = '%';
		    *optr++ = toHex(*dptr>>4);
		    *optr++ = toHex(*dptr%16);
		}
	    dptr++;
	}
	*optr = '\0';
	//*optr++ = '\0';
	delete[] data;
	data = obuf;
*/
	while (*dptr)
	{
		if (((*dptr >= 'a') && (*dptr <= 'z')) ||
			((*dptr >= 'A') && (*dptr <= 'Z')) ||
			((*dptr >= '0') && (*dptr <= '9')) ||
			(*dptr == '/') || (*dptr == '-') || (*dptr == ':'))
		{
			*optr++ = *dptr++;
		}
		else
		{
			*optr++ = '%';
			*optr++ = hdig[(unsigned char)(*dptr) >> 4];
			*optr++ = hdig[(unsigned char)(*dptr) & 15];
			dptr++;
		}
	}
	*optr++ = '\0';
	delete[] data;
	data = obuf;

}


// Get*
//
// get an item; advance ls past end of item

// get line terminated by [<CR>]<LF>

char* GetLine(const char*& ls)
{
	const char*	le = ls;
	while (*le && (*le != '\n'))		le++;

	char* line = StringDup(ls, le);

	// remove <CR>
	if ((le > ls) && (le[-1] == 0x0D))	line[le - ls - 1] = 0;
	if (*le)	le++;
	ls = le;

	return line;
}

char* GetAlphaNumeric(const char*& ls)
{
	const char*	le = ls;
	while (isalnum(*le))	le++;

	char* word = StringDup(ls, le);

	ls = le;

	return word;
}

//K+
char* GetSquare(const char*& ls)
{
	ls++;

	const char*	le = ls;

	while (*le!=']') {
		if (*le=='[') return NULL;
		le++;
		if (!*le) return NULL;
	}
	char* word = StringDup(ls, le);

	ls = le+1;

	return word;
}

//K+
void Utf8ToAscii(char*& data)
{
	BSTR bstr;
	int nLen = MultiByteToWideChar(CP_UTF8, 0, data, lstrlen(data), NULL, NULL);
	bstr = SysAllocStringLen(NULL, nLen);
	MultiByteToWideChar(CP_UTF8, 0, data, lstrlen(data), bstr, nLen);

	char *result;
	nLen = WideCharToMultiByte(CP_ACP, 0, bstr, -1, NULL, 0, NULL, NULL);
	result = (char *) malloc(nLen+1);
	WideCharToMultiByte(CP_ACP, 0, bstr, -1, result, 128, NULL, NULL);
	data = result;

	SysFreeString(bstr);
}

//K+
void AsciiToUtf8(char*& data)
{
	BSTR bstr;
	int nLen = MultiByteToWideChar(CP_ACP, 0, data, lstrlen(data), NULL, NULL);
	bstr = SysAllocStringLen(NULL, nLen);
	MultiByteToWideChar(CP_ACP, 0, data, lstrlen(data), bstr, nLen);

	char *result;
	nLen = WideCharToMultiByte(CP_UTF8, 0, bstr, -1, NULL, 0, NULL, NULL);
	result = (char *) malloc(nLen+1);
	WideCharToMultiByte(CP_UTF8, 0, bstr, -1, result, 128, NULL, NULL);
	data = result;

	SysFreeString(bstr);
}


// get valid URL

static bool isurl(char c)
{
	// leaf characters
	if (isalnum(c) || (c == '-') || (c == '_'))	return true;
	// URL seperators
	if ((c == ':') || (c == '/') || (c == '.') || (c == '#'))	return true;
	// CGI access characters
	if ((c == '+') || (c == '%') || (c == '?') || (c == '&') || (c == '='))	return true;
	// email characters
	if (c == '@')	return true;
	// UNIX special characters
	if (c == '~')	return true;

	return false;
}

char* GetURL(const char*& ls)
{
	const char*	le = ls;
	while (isurl(*le))	le++;

	// copy URL
	char*	url = StringDup(ls, le);

	ls = le;
	
	return url;
}

char* GetNumber(const char*& ls)
{
	const char*	le = ls;
	while (isdigit(*le))	le++;

	char*	num = StringDup(ls, le);

	ls = le;

	return num;
}

// get ISBN - ignore leading spaces, allow - anywhere,
// last digit can be X ... returns NULL if invalid
//
// also copies raw ISBN (just digits) into rawisbn

char* GetISBN(const char*& ls, char rawisbn[11])
{
	const char* le = ls;
	if (*le != ' ')	return NULL;
	le++;

	int		ix = 0;

	for (; *le; le++)
	{
		if (*le == '-')		continue;

		if ((*le >= '0') && (*le <= '9'))
		{
			rawisbn[ix] = *le;
			ix++;
			if (ix == 11)	return NULL;
		}
		else if ((*le == 'X') || (*le == 'x'))
		{
			if (ix != 9)	return NULL;
			rawisbn[ix] = *le;
			ix++;
		}
		else break;
	}

	if (ix != 10)	return NULL;
	rawisbn[10] = 0;

	char*	isbn = StringDup(ls, le);

	ls = le;

	return isbn;
}

const char* MonthName(int ix)
{
	static char* month[] = { "January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December" };

	if ((ix < 1) || (ix > 12))	return "";
	
	return month[ix - 1];
}

// MatchWildcardString
//
// see if a wildcard string is within another string

bool MatchWildcardString(const char* string, const char* search, bool recursing)
{
	// scan along string
	while (string[0])
	{
		if (recursing && (string[0] == '\n'))	return false;	// don't let * go over \n

		const char*	sptr = string;
		const char*	wptr = search;

		// scan along search string
		while (*wptr)
		{
			if (*wptr == '*')	return MatchWildcardString(sptr, wptr + 1, true);
			if (((*wptr == '?') && (*sptr != '\n')) ||			// ? matches anything but \n
				(toupper(*wptr) == toupper(*sptr)))				// not case sensitive
			{
				wptr++;
				sptr++;
			}
			else break;
		}

		if (!*wptr)	return true;

		string++;
	}

	return false;
}
